@@ -5,9 +5,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifndef _MSC_VER
+
+#ifdef HAVE_UNISTD_H
 # include <unistd.h>
-#endif /* ifndef _MSC_VER */
+#endif /* ifdef HAVE_UNISTD_H */
 
 #ifdef HAVE_EVIL
 # include <Evil.h>
@@ -43,11 +44,11 @@ static int _eet_main_log_dom = -1;
 #define CRIT(...) EINA_LOG_DOM_CRIT(_eet_main_log_dom, __VA_ARGS__)
 
 static void
-do_eet_list(const char * file)
+do_eet_list(const char *file)
 {
    int i, num;
-   char ** list;
-   Eet_File * ef;
+   char **list;
+   Eet_File *ef;
 
    ef = eet_open(file, EET_FILE_MODE_READ);
    if (!ef)
@@ -60,7 +61,7 @@ do_eet_list(const char * file)
    if (list)
      {
         for (i = 0; i < num; i++)
-           printf("%s\n",list[i]);
+          printf("%s\n", list[i]);
         free(list);
      }
 
@@ -68,15 +69,72 @@ do_eet_list(const char * file)
 } /* do_eet_list */
 
 static void
-do_eet_extract(const char * file,
-               const char * key,
-               const char * out,
-               const char * crypto_key)
+do_eet_stats(const char *file)
 {
-   Eet_File * ef;
-   void * data;
+   int i, num;
+   int count[2] = { 0, 0 };
+   int size[2] = { 0, 0 };
+   char **list;
+   Eet_File *ef;
+   Eet_Dictionary *ed;
+
+   ef = eet_open(file, EET_FILE_MODE_READ);
+   if (!ef)
+     {
+        ERR("cannot open for reading: %s", file);
+        exit(-1);
+     }
+
+   printf("*** sections stats ***\n");
+   list = eet_list(ef, "*", &num);
+   if (list)
+     {
+        for (i = 0; i < num; i++)
+          {
+             const void *ro = NULL;
+             void *rw = NULL;
+             int tsize;
+
+             ro = eet_read_direct(ef, list[i], &tsize);
+             if (!ro) rw = eet_read(ef, list[i], &tsize);
+             printf(rw ? "%s of size %i is compressed.\n" : "%s of size %i is not compressed.\n", list[i], tsize);
+             count[rw ? 0 : 1]++;
+             size[rw ? 0 : 1] += tsize;
+             free(rw);
+          }
+        free(list);
+     }
+
+   printf("*** dictionary ***\n");
+   ed = eet_dictionary_get(ef);
+   if (ed)
+     {
+        printf("%i strings inside the dictionary.\n", eet_dictionary_count(ed));
+     }
+   else
+     {
+        printf("no dictionary in this file.\n");
+     }
+   printf("*** global ***\n");
+   printf("%i sections\n", num);
+   printf("- %i of them are compressed (%02.2f%%) expanding in %i bytes.\n",
+          count[0], (float) count[0] * 100 / (float) num, size[0]);
+   printf("- %i of them are directly mappable in memory (%02.2f%%) representing %i bytes.\n",
+          count[1], (float) count[1] * 100 / (float) num, size[1]);
+
+   eet_close(ef);
+}
+
+static void
+do_eet_extract(const char *file,
+               const char *key,
+               const char *out,
+               const char *crypto_key)
+{
+   Eet_File *ef;
+   void *data;
    int size = 0;
-   FILE * f;
+   FILE *f = stdout;
 
    ef = eet_open(file, EET_FILE_MODE_READ);
    if (!ef)
@@ -92,38 +150,42 @@ do_eet_extract(const char * file,
         exit(-1);
      }
 
-   f = fopen(out, "wb");
-   if (!f)
+   if (out)
      {
-        ERR("cannot open %s", out);
-        exit(-1);
+        f = fopen(out, "wb");
+        if (!f)
+          {
+             ERR("cannot open %s", out);
+             exit(-1);
+          }
      }
 
    if (fwrite(data, size, 1, f) != 1)
      {
-        ERR("cannot write to %s", out);
+        ERR("cannot write to %s", out ? out : "standard output");
         exit(-1);
      }
 
-   fclose(f);
+   if (out) fclose(f);
    free(data);
    eet_close(ef);
 } /* do_eet_extract */
 
 static void
-do_eet_decode_dump(void * data, const char * str)
+do_eet_decode_dump(void       *data,
+                   const char *str)
 {
    fputs(str, (FILE *)data);
 } /* do_eet_decode_dump */
 
 static void
-do_eet_decode(const char * file,
-              const char * key,
-              const char * out,
-              const char * crypto_key)
+do_eet_decode(const char *file,
+              const char *key,
+              const char *out,
+              const char *crypto_key)
 {
-   Eet_File * ef;
-   FILE * f;
+   Eet_File *ef;
+   FILE *f = stdout;
 
    ef = eet_open(file, EET_FILE_MODE_READ);
    if (!ef)
@@ -132,38 +194,41 @@ do_eet_decode(const char * file,
         exit(-1);
      }
 
-   f = fopen(out, "wb");
-   if (!f)
+   if (out)
      {
-        ERR("cannot open %s", out);
-        exit(-1);
+        f = fopen(out, "wb");
+        if (!f)
+          {
+             ERR("cannot open %s", out);
+             exit(-1);
+          }
      }
 
    if (!eet_data_dump_cipher(ef, key, crypto_key, do_eet_decode_dump, f))
      {
-        ERR("cannot write to %s", out);
+        ERR("cannot write to %s", out ? out : "standard output");
         exit(-1);
      }
 
-   fclose(f);
+   if (out) fclose(f);
    eet_close(ef);
 } /* do_eet_decode */
 
 static void
-do_eet_insert(const char * file,
-              const char * key,
-              const char * out,
-              int          compress,
-              const char * crypto_key)
+do_eet_insert(const char *file,
+              const char *key,
+              const char *out,
+              int         compress,
+              const char *crypto_key)
 {
-   Eet_File * ef;
-   void * data;
+   Eet_File *ef;
+   void *data;
    int size = 0;
-   FILE * f;
+   FILE *f;
 
    ef = eet_open(file, EET_FILE_MODE_READ_WRITE);
    if (!ef)
-      ef = eet_open(file, EET_FILE_MODE_WRITE);
+     ef = eet_open(file, EET_FILE_MODE_WRITE);
 
    if (!ef)
      {
@@ -201,21 +266,21 @@ do_eet_insert(const char * file,
 } /* do_eet_insert */
 
 static void
-do_eet_encode(const char * file,
-              const char * key,
-              const char * out,
-              int          compress,
-              const char * crypto_key)
+do_eet_encode(const char *file,
+              const char *key,
+              const char *out,
+              int         compress,
+              const char *crypto_key)
 {
-   Eet_File * ef;
-   char * text;
+   Eet_File *ef;
+   char *text;
    int textlen = 0;
    int size = 0;
-   FILE * f;
+   FILE *f;
 
    ef = eet_open(file, EET_FILE_MODE_READ_WRITE);
    if (!ef)
-      ef = eet_open(file, EET_FILE_MODE_WRITE);
+     ef = eet_open(file, EET_FILE_MODE_WRITE);
 
    if (!ef)
      {
@@ -258,9 +323,10 @@ do_eet_encode(const char * file,
 } /* do_eet_encode */
 
 static void
-do_eet_remove(const char * file, const char * key)
+do_eet_remove(const char *file,
+              const char *key)
 {
-   Eet_File * ef;
+   Eet_File *ef;
 
    ef = eet_open(file, EET_FILE_MODE_READ_WRITE);
    if (!ef)
@@ -274,10 +340,10 @@ do_eet_remove(const char * file, const char * key)
 } /* do_eet_remove */
 
 static void
-do_eet_check(const char * file)
+do_eet_check(const char *file)
 {
-   Eet_File * ef;
-   const void * der;
+   Eet_File *ef;
+   const void *der;
    int der_length;
    int sign_length;
 
@@ -300,10 +366,12 @@ do_eet_check(const char * file)
 } /* do_eet_check */
 
 static void
-do_eet_sign(const char * file, const char * private_key, const char * public_key)
+do_eet_sign(const char *file,
+            const char *private_key,
+            const char *public_key)
 {
-   Eet_File * ef;
-   Eet_Key * key;
+   Eet_File *ef;
+   Eet_Key *key;
 
    ef = eet_open(file, EET_FILE_MODE_READ_WRITE);
    if (!ef)
@@ -328,77 +396,109 @@ do_eet_sign(const char * file, const char * private_key, const char * public_key
 } /* do_eet_sign */
 
 int
-main(int argc, char ** argv)
+main(int    argc,
+     char **argv)
 {
    if (!eet_init())
-      return -1;
+     return -1;
 
    _eet_main_log_dom = eina_log_domain_register("eet_main", EINA_COLOR_CYAN);
    if(_eet_main_log_dom < -1)
      {
         EINA_LOG_ERR("Impossible to create a log domain for eet_main.");
         eet_shutdown();
-        return(-1);
+        return -1;
      }
 
    if (argc < 2)
      {
 help:
         printf(
-           "Usage:\n"
-           "  eet -l FILE.EET				     list all keys in FILE.EET\n"
-           "  eet -x FILE.EET KEY OUT-FILE [CRYPTO_KEY]          extract data stored in KEY in FILE.EET and write to OUT-FILE\n"
-           "  eet -d FILE.EET KEY OUT-FILE [CRYPTO_KEY]          extract and decode data stored in KEY in FILE.EET and write to OUT-FILE\n"
-           "  eet -i FILE.EET KEY IN-FILE COMPRESS [CRYPTO_KEY]  insert data to KEY in FILE.EET from IN-FILE and if COMPRESS is 1, compress it\n"
-           "  eet -e FILE.EET KEY IN-FILE COMPRESS [CRYPTO_KEY]  insert and encode to KEY in FILE.EET from IN-FILE and if COMPRESS is 1, compress it\n"
-           "  eet -r FILE.EET KEY                                remove KEY in FILE.EET\n"
-           "  eet -c FILE.EET                                    report and check the signature information of an eet file\n"
-           "  eet -s FILE.EET PRIVATE_KEY PUBLIC_KEY             sign FILE.EET with PRIVATE_KEY and attach PUBLIC_KEY as it's certificate\n"
-           );
+          "Usage:\n"
+          "  eet -l FILE.EET                                    list all keys in FILE.EET\n"
+          "  eet -x FILE.EET KEY [OUT-FILE] [CRYPTO_KEY]        extract data stored in KEY in FILE.EET and write to OUT-FILE or standard output\n"
+          "  eet -d FILE.EET KEY [OUT-FILE] [CRYPTO_KEY]        extract and decode data stored in KEY in FILE.EET and write to OUT-FILE or standard output\n"
+          "  eet -i FILE.EET KEY IN-FILE COMPRESS [CRYPTO_KEY]  insert data to KEY in FILE.EET from IN-FILE and if COMPRESS is 1, compress it\n"
+          "  eet -e FILE.EET KEY IN-FILE COMPRESS [CRYPTO_KEY]  insert and encode to KEY in FILE.EET from IN-FILE and if COMPRESS is 1, compress it\n"
+          "  eet -r FILE.EET KEY                                remove KEY in FILE.EET\n"
+          "  eet -c FILE.EET                                    report and check the signature information of an eet file\n"
+          "  eet -s FILE.EET PRIVATE_KEY PUBLIC_KEY             sign FILE.EET with PRIVATE_KEY and attach PUBLIC_KEY as it's certificate\n"
+          "  eet -t FILE.EET                                    give some statistic about a file\n"
+          );
         eet_shutdown();
         return -1;
      }
 
    if ((!strncmp(argv[1], "-h", 2)))
-      goto help;
+     goto help;
    else if ((!strcmp(argv[1], "-l")) && (argc > 2))
-      do_eet_list(argv[2]);
-   else if ((!strcmp(argv[1], "-x")) && (argc > 4))
+     do_eet_list(argv[2]);
+   else if ((!strcmp(argv[1], "-x")) && (argc > 3))
      {
-        if (argc > 5)
-           do_eet_extract(argv[2], argv[3], argv[4], argv[5]);
-        else
-           do_eet_extract(argv[2], argv[3], argv[4], NULL);
+        switch (argc)
+          {
+           case 4:
+             {
+               do_eet_extract(argv[2], argv[3], NULL, NULL);
+               break;
+             }
+           case 5:
+             {
+               do_eet_extract(argv[2], argv[3], argv[4], NULL);
+               break;
+             }
+           default:
+             {
+               do_eet_extract(argv[2], argv[3], argv[4], argv[5]);
+               break;
+             }
+          }
      }
-   else if ((!strcmp(argv[1], "-d")) && (argc > 4))
+   else if ((!strcmp(argv[1], "-d")) && (argc > 3))
      {
-        if (argc > 5)
-           do_eet_decode(argv[2], argv[3], argv[4], argv[5]);
-        else
-           do_eet_decode(argv[2], argv[3], argv[4], NULL);
+        switch (argc)
+          {
+           case 4:
+             {
+               do_eet_decode(argv[2], argv[3], NULL, NULL);
+               break;
+             }
+           case 5:
+             {
+               do_eet_decode(argv[2], argv[3], argv[4], NULL);
+               break;
+             }
+           default:
+             {
+               do_eet_decode(argv[2], argv[3], argv[4], argv[5]);
+               break;
+             }
+          }
      }
    else if ((!strcmp(argv[1], "-i")) && (argc > 5))
      {
         if (argc > 6)
-           do_eet_insert(argv[2], argv[3], argv[4], atoi(argv[5]), argv[6]);
+          do_eet_insert(argv[2], argv[3], argv[4], atoi(argv[5]), argv[6]);
         else
-           do_eet_insert(argv[2], argv[3], argv[4], atoi(argv[5]), NULL);
+          do_eet_insert(argv[2], argv[3], argv[4], atoi(argv[5]), NULL);
      }
    else if ((!strcmp(argv[1], "-e")) && (argc > 5))
      {
         if (argc > 6)
-           do_eet_encode(argv[2], argv[3], argv[4], atoi(argv[5]), argv[6]);
+          do_eet_encode(argv[2], argv[3], argv[4], atoi(argv[5]), argv[6]);
         else
-           do_eet_encode(argv[2], argv[3], argv[4], atoi(argv[5]), NULL);
+          do_eet_encode(argv[2], argv[3], argv[4], atoi(argv[5]), NULL);
      }
    else if ((!strcmp(argv[1], "-r")) && (argc > 3))
-      do_eet_remove(argv[2], argv[3]);
+     do_eet_remove(argv[2], argv[3]);
    else if ((!strcmp(argv[1], "-c")) && (argc > 2))
-      do_eet_check(argv[2]);
+     do_eet_check(argv[2]);
    else if ((!strcmp(argv[1], "-s")) && (argc > 4))
-      do_eet_sign(argv[2], argv[3], argv[4]);
+     do_eet_sign(argv[2], argv[3], argv[4]);
+   else if ((!strcmp(argv[1], "-t")) && (argc > 2))
+     do_eet_stats(argv[2]);
    else
-      goto help;
+     goto help;
 
    eina_log_domain_unregister(_eet_main_log_dom);
    eet_shutdown();
