@@ -706,9 +706,13 @@ eet_identity_check(const void   *data_base,
      return NULL;
 
    /* Get the header */
-   magic = ntohl(header[0]);
-   sign_len = ntohl(header[1]);
-   cert_len = ntohl(header[2]);
+   memcpy(&magic,    header, sizeof(int));
+   memcpy(&sign_len, header+1, sizeof(int));
+   memcpy(&cert_len, header+2, sizeof(int));
+
+   magic = ntohl(magic);
+   sign_len = ntohl(sign_len);
+   cert_len = ntohl(cert_len);
 
    /* Verify the header */
    if (magic != EET_MAGIC_SIGN)
@@ -968,7 +972,7 @@ eet_cipher(const void   *data,
 # else /* ifdef HAVE_GNUTLS */
    /* Openssl declarations*/
    EVP_CIPHER_CTX ctx;
-   unsigned int *buffer;
+   unsigned int *buffer = NULL;
    int tmp_len;
 # endif /* ifdef HAVE_GNUTLS */
 
@@ -1043,7 +1047,8 @@ eet_cipher(const void   *data,
    /* Gcrypt close the cipher */
    gcry_cipher_close(cipher);
 # else /* ifdef HAVE_GNUTLS */
-   buffer = alloca(crypted_length);
+   buffer = malloc(crypted_length);
+   if (!buffer) goto on_error;
    *buffer = tmp;
 
    memcpy(buffer + 1, data, size);
@@ -1071,6 +1076,7 @@ eet_cipher(const void   *data,
      goto on_error;
 
    EVP_CIPHER_CTX_cleanup(&ctx);
+   free(buffer);
 # endif /* ifdef HAVE_GNUTLS */
 
    /* Set return values */
@@ -1098,6 +1104,8 @@ on_error:
    if (opened)
      EVP_CIPHER_CTX_cleanup(&ctx);
 
+   free(buffer);
+   
 # endif /* ifdef HAVE_GNUTLS */
    /* General error */
    free(ret);
@@ -1219,7 +1227,7 @@ eet_decipher(const void   *data,
    /* Get the decrypted data size */
    tmp = *ret;
    tmp = ntohl(tmp);
-   if (tmp > tmp_len)
+   if (tmp > tmp_len || tmp <= 0)
      goto on_error;
 
    /* Update the return values */
@@ -1375,12 +1383,13 @@ eet_pbkdf2_sha1(const char          *key,
              for (k = 0; k < tmp_len; k++)
                p[k] ^= digest[k];
           }
-     }
 
 # ifdef HAVE_GNUTLS
 # else
-   HMAC_cleanup(&hctx);
+	HMAC_cleanup(&hctx);
 # endif /* ifdef HAVE_GNUTLS */
+     }
+
    return 0;
 }
 
